@@ -9,7 +9,7 @@
 -- |
 -- Module:      Database.PostgreSQL.Simple
 -- Copyright:   (c) 2011 MailRank, Inc.
---              (c) 2011 Leon P Smith
+--              (c) 2011-2012 Leon P Smith
 -- License:     BSD3
 -- Maintainer:  Leon P Smith <leon@melding-monads.com>
 -- Stability:   experimental
@@ -43,6 +43,9 @@ module Database.PostgreSQL.Simple
     -- ** Modifying multiple rows at once
     -- $many
 
+    -- ** @RETURNING@: modifications that returns results
+    -- $returning
+
     -- * Extracting results
     -- $result
 
@@ -56,6 +59,8 @@ module Database.PostgreSQL.Simple
       Base.ConnectInfo(..)
     , Connection
     , Query
+    , ToRow
+    , FromRow
     , In(..)
     , Binary(..)
     , Only(..)
@@ -84,6 +89,7 @@ module Database.PostgreSQL.Simple
     , foldWithOptions_
     , forEach
     , forEach_
+    , returning
     -- * Statements that do not return results
     , execute
     , execute_
@@ -91,6 +97,7 @@ module Database.PostgreSQL.Simple
 --    , Base.insertID
     -- * Transaction handling
     , withTransaction
+    , withTransactionSerializable
     , TransactionMode(..)
     , IsolationLevel(..)
     , ReadWriteMode(..)
@@ -99,6 +106,7 @@ module Database.PostgreSQL.Simple
     , defaultReadWriteMode
     , withTransactionLevel
     , withTransactionMode
+    , withTransactionModeRetry
 --    , Base.autocommit
     , begin
     , beginLevel
@@ -116,7 +124,8 @@ import           Blaze.ByteString.Builder.Char8 (fromChar)
 import           Control.Applicative ((<$>), pure)
 import           Control.Concurrent.MVar
 import           Control.Exception
-                   ( Exception, onException, throw, throwIO, finally )
+                   ( Exception, onException, throw, throwIO, finally
+                   , try, SomeException, fromException )
 import           Control.Monad (foldM)
 import           Data.ByteString (ByteString)
 import           Data.Int (Int64)
@@ -124,8 +133,7 @@ import qualified Data.IntMap as IntMap
 import           Data.List (intersperse)
 import           Data.Monoid (mappend, mconcat)
 import           Data.Typeable (Typeable)
-import           Database.PostgreSQL.Simple.BuiltinTypes
-                   ( oid2builtin, builtin2typname )
+import           Database.PostgreSQL.Simple.BuiltinTypes ( oid2typname )
 import           Database.PostgreSQL.Simple.Compat ( mask )
 import           Database.PostgreSQL.Simple.FromField (ResultError(..))
 import           Database.PostgreSQL.Simple.FromRow (FromRow(..))
@@ -192,7 +200,7 @@ formatMany conn q@(Query template) qs = do
       return . toByteString . mconcat $ fromByteString before :
                                         intersperse (fromChar ',') bs ++
                                         [fromByteString after]
-    Nothing -> fmtError "syntax error in query template for executeMany" q []
+    Nothing -> fmtError "syntax error in multi-row template" q []
 
 -- Split the input string into three pieces, @before@, @qbits@, and @after@,
 -- following this grammar:
@@ -310,7 +318,7 @@ buildQuery conn q template xs = zipParams (split template) <$> mapM sub xs
         utf8ToString = T.unpack . TE.decodeUtf8
         sub (Plain  b)      = pure b
         sub (Escape s)      = quote <$> escapeStringConn conn s
-        sub (EscapeBytea s) = quote <$> escapeByteaConn conn s
+        sub (EscapeByteA s) = quote <$> escapeByteaConn conn s
         sub (Many  ys)      = mconcat <$> mapM sub ys
         split s = fromByteString h : if B.null t then [] else split (B.tail t)
             where (h,t) = B.break (=='?') s
@@ -343,6 +351,20 @@ executeMany conn q qs = do
   result <- exec conn =<< formatMany conn q qs
   finishExecute conn q result
 
+-- | Execute @INSERT ... RETURNING@, @UPDATE ... RETURNING@, or other SQL
+-- query that accepts multi-row input and is expected to return results.
+-- Note that it is possible to write
+--    @'query' conn "INSERT ... RETURNING ..." ...@
+-- in cases where you are only inserting a single row,  and do not need
+-- functionality analogous to 'executeMany'.
+--
+-- Throws 'FormatError' if the query could not be formatted correctly.
+returning :: (ToRow q, FromRow r) => Connection -> Query -> [q] -> IO [r]
+returning _ _ [] = return []
+returning conn q qs = do
+  result <- exec conn =<< formatMany conn q qs
+  finishQuery conn q result
+
 -- | Perform a @SELECT@ or other SQL query that is expected to return
 -- results. All results are retrieved and converted before this
 -- function returns.
@@ -624,6 +646,24 @@ defaultReadWriteMode   =  DefaultReadWriteMode
 withTransaction :: Connection -> IO a -> IO a
 withTransaction = withTransactionMode defaultTransactionMode
 
+-- | Execute an action inside of a 'Serializable' transaction.  If a
+-- serialization failure occurs, roll back the transaction and try again.
+-- Be warned that this may execute the IO action multiple times.
+--
+-- A 'Serializable' transaction creates the illusion that your program has
+-- exclusive access to the database.  This means that, even in a concurrent
+-- setting, you can perform queries in sequence without having to worry about
+-- what might happen between one statement and the next.
+--
+-- Think of it as STM, but without @retry@.
+withTransactionSerializable :: Connection -> IO a -> IO a
+withTransactionSerializable =
+    withTransactionModeRetry
+        TransactionMode
+        { isolationLevel = Serializable
+        , readWriteMode  = ReadWrite
+        }
+
 -- | Execute an action inside a SQL transaction with a given isolation level.
 withTransactionLevel :: IsolationLevel -> Connection -> IO a -> IO a
 withTransactionLevel lvl
@@ -638,6 +678,37 @@ withTransactionMode mode conn act =
     commit conn
     return r
 
+-- | Like 'withTransactionMode', but if a 'SqlError' arises whose 'sqlState' is
+-- @\"40001\"@ (@serialization_failure@), this will issue a @ROLLBACK@, then
+-- try the action again.  If any other exception arises, this will issue a
+-- @ROLLBACK@, but will propagate the exception instead of retrying.
+--
+-- This is used to implement 'withTransactionSerializable'.
+withTransactionModeRetry :: TransactionMode -> Connection -> IO a -> IO a
+withTransactionModeRetry mode conn act =
+    mask $ \restore ->
+        retryLoop $ try $ do
+            a <- restore act
+            commit conn
+            return a
+  where
+    retryLoop :: IO (Either SomeException a) -> IO a
+    retryLoop act' = do
+        beginMode mode conn
+        r <- act'
+        case r of
+            Left e -> do
+                rollback conn
+                case fromException e of
+                    Just SqlError{..} | sqlState == serialization_failure
+                      -> retryLoop act'
+                    _ -> throwIO e
+            Right a ->
+                return a
+
+    -- http://www.postgresql.org/docs/current/static/errcodes-appendix.html
+    serialization_failure = "40001"
+
 -- | Rollback a transaction.
 rollback :: Connection -> IO ()
 rollback conn = execute_ conn "ABORT" >> return ()
@@ -678,7 +749,7 @@ fmtError msg q xs = throw FormatError {
                     }
   where twiddle (Plain b)       = toByteString b
         twiddle (Escape s)      = s
-        twiddle (EscapeBytea s) = s
+        twiddle (EscapeByteA s) = s
         twiddle (Many ys)       = B.concat (map twiddle ys)
 
 -- $use
@@ -855,6 +926,20 @@ fmtError msg q xs = throw FormatError {
 -- > insert into users (first_name,last_name) values
 -- >   ('Boris','Karloff'),('Ed','Wood')
 
+-- $returning
+--
+-- The 'returning' function is similar to 'executeMany' but is
+-- intended for use with multi-tuple @INSERT@ or @UPDATE@ statements
+-- that make use of @RETURNING@.
+--
+-- For example, were there an auto-incrementing @id@ column and
+-- timestamp column @t@ that defaulted to the present time for the
+-- @sales@ table, then the following query would insert two new
+-- sales records and also return their new @id@s and timestamps.
+--
+-- > let q = "insert into sales (amount, label) values (?,?) returning id, t"
+-- > xs :: [(Int, UTCTime)] <- returning conn q [(20,"Chips"),(300,"Wood")]
+
 -- $result
 --
 -- The 'query' and 'query_' functions return a list of values in the
@@ -932,8 +1017,8 @@ fmtError msg q xs = throw FormatError {
 
 getTypename :: Connection -> PQ.Oid -> IO ByteString
 getTypename conn@Connection{..} oid =
-  case oid2builtin oid of
-    Just builtin -> return $! builtin2typname builtin
+  case oid2typname oid of
+    Just name -> return name
     Nothing -> modifyMVar connectionObjects $ \oidmap -> do
       case IntMap.lookup (oid2int oid) oidmap of
         Just name -> return (oidmap, name)
