@@ -7,6 +7,7 @@ import System.IO
 import Bytea
 import ExecuteMany
 import Notify
+import Serializable
 import Time
 
 tests :: [TestEnv -> Test]
@@ -15,6 +16,7 @@ tests =
     , TestLabel "Notify"        . testNotify
     , TestLabel "ExecuteMany"   . testExecuteMany
     , TestLabel "Time"          . testTime
+    , TestLabel "Serializable"  . testSerializable
     ]
 
 -- | Action for connecting to the database that will be used for testing.
